package sun.rmi.server;

import sun.misc.ObjectStreamClassValidator;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.misc.SharedSecrets;
import java.security.Permission;
import sun.misc.VM;
import java.security.AccessControlException;
import java.rmi.server.RMIClassLoader;
import java.io.ObjectStreamClass;
import java.util.Iterator;
import java.io.StreamCorruptedException;
import java.io.IOException;
import java.util.HashMap;
import java.io.InputStream;
import java.util.Map;
import java.io.ObjectInputStream;

public class MarshalInputStream extends ObjectInputStream
{
    private volatile MarshalInputStream.StreamChecker streamChecker;
    private static final boolean useCodebaseOnlyProperty;
    protected static Map<String, Class<?>> permittedSunClasses;
    private boolean skipDefaultResolveClass;
    private final Map<Object, Runnable> doneCallbacks;
    private boolean useCodebaseOnly;
    
    public MarshalInputStream(InputStream in) throws IOException, StreamCorruptedException {
        super(in);
        this.streamChecker = null;
        this.skipDefaultResolveClass = false;
        this.doneCallbacks = new HashMap(3);
        this.useCodebaseOnly = MarshalInputStream.useCodebaseOnlyProperty;
    }
    
    public Runnable getDoneCallback(Object o) {
        return (Runnable)this.doneCallbacks.get(o);
    }
    
    public void setDoneCallback(Object o, Runnable runnable) {
        this.doneCallbacks.put(o, runnable);
    }
    
    public void done() {
        Iterator<Runnable> iterator = this.doneCallbacks.values().iterator();
        while (iterator.hasNext()) {
            ((Runnable)iterator.next()).run();
        }
        this.doneCallbacks.clear();
    }
    
    @Override
    public void close() throws IOException {
        this.done();
        super.close();
    }
    
    @Override
    protected Class<?> resolveClass(ObjectStreamClass desc) throws IOException, ClassNotFoundException {
        Object location = this.readLocation();
        String name = desc.getName();
        ClassLoader defaultLoader = this.skipDefaultResolveClass ? null : latestUserDefinedLoader();
        String codebase = null;
        if (!this.useCodebaseOnly && location instanceof String) {
            codebase = (String)location;
        }
        try {
            return RMIClassLoader.loadClass(codebase, name, defaultLoader);
        }
        catch (AccessControlException ex) {
            return this.checkSunClass(name, ex);
        }
        catch (ClassNotFoundException ex2) {
            try {
                if (Character.isLowerCase(name.charAt(0)) && name.indexOf(46) == -1) {
                    return super.resolveClass(desc);
                }
            }
            catch (ClassNotFoundException ex3) {}
            throw ex2;
        }
    }
    
    @Override
    protected Class<?> resolveProxyClass(String[] interfaces) throws IOException, ClassNotFoundException {
        MarshalInputStream.StreamChecker streamChecker = this.streamChecker;
        if (streamChecker != null) {
            streamChecker.checkProxyInterfaceNames(interfaces);
        }
        Object location = this.readLocation();
        ClassLoader defaultLoader = this.skipDefaultResolveClass ? null : latestUserDefinedLoader();
        String codebase = null;
        if (!this.useCodebaseOnly && location instanceof String) {
            codebase = (String)location;
        }
        return RMIClassLoader.loadProxyClass(codebase, interfaces, defaultLoader);
    }
    
    private static ClassLoader latestUserDefinedLoader() {
        return VM.latestUserDefinedLoader();
    }
    
    private Class<?> checkSunClass(String s, AccessControlException ex) throws AccessControlException {
        Permission permission = ex.getPermission();
        String name = null;
        if (permission != null) {
            name = permission.getName();
        }
        Class<Object> clazz = (Class)MarshalInputStream.permittedSunClasses.get(s);
        if (name == null || clazz == null || (!name.equals("accessClassInPackage.sun.rmi.server") && !name.equals("accessClassInPackage.sun.rmi.registry"))) {
            throw ex;
        }
        return clazz;
    }
    
    protected Object readLocation() throws IOException, ClassNotFoundException {
        return this.readObject();
    }
    
    void skipDefaultResolveClass() {
        this.skipDefaultResolveClass = true;
    }
    
    void useCodebaseOnly() {
        this.useCodebaseOnly = true;
    }
    
    synchronized void setStreamChecker(MarshalInputStream.StreamChecker streamChecker) {
        this.streamChecker = streamChecker;
        SharedSecrets.getJavaObjectInputStreamAccess().setValidator(this, streamChecker);
    }
    
    @Override
    protected ObjectStreamClass readClassDescriptor() throws IOException, ClassNotFoundException {
        ObjectStreamClass classDescriptor = super.readClassDescriptor();
        this.validateDesc(classDescriptor);
        return classDescriptor;
    }
    
    private void validateDesc(ObjectStreamClass objectStreamClass) {
        MarshalInputStream.StreamChecker streamChecker;
        synchronized (this) {
            streamChecker = this.streamChecker;
        }
        if (streamChecker != null) {
            streamChecker.validateDescriptor(objectStreamClass);
        }
    }
    
    static {
        useCodebaseOnlyProperty = !((String)AccessController.doPrivileged(new GetPropertyAction("java.rmi.server.useCodebaseOnly", "true"))).equalsIgnoreCase("false");
        MarshalInputStream.permittedSunClasses = new HashMap(3);
        try {
            String className = "sun.rmi.server.Activation$ActivationSystemImpl_Stub";
            String className2 = "sun.rmi.registry.RegistryImpl_Stub";
            MarshalInputStream.permittedSunClasses.put(className, Class.forName(className));
            MarshalInputStream.permittedSunClasses.put(className2, Class.forName(className2));
        }
        catch (ClassNotFoundException ex) {
            throw new NoClassDefFoundError("Missing system class: " + ex.getMessage());
        }
    }
    
    interface StreamChecker extends ObjectStreamClassValidator
    {
        public abstract void checkProxyInterfaceNames(String[] p0);
    }
}
